ConfBuilder

Struct ConfBuilder 

pub struct ConfBuilder<S, F = fn(&dyn ConfigEvent)>
where S: Conf, F: FnMut(&dyn ConfigEvent),
{ /* private fields */ }
A builder which collects config value sources and other options, before parsing your struct.

If args is not set, the default source is std::env::args_os. If env is not set, the default source is std::env::vars_os.

Implementations

Source

impl<S, F> ConfBuilder<S, F>
where S: Conf, F: FnMut(&dyn ConfigEvent),

pub fn args(self, args: impl IntoIterator<Item: Into<OsString>>) -> Self

Set the CLI args used in this parse

pub fn env<K, V>(self, env: impl IntoIterator<Item = (K, V)>) -> Self
where K: Into<OsString>, V: Into<OsString>,

Set the env vars used in this parse

pub fn config_logger<F2: FnMut(&dyn ConfigEvent)>( self, f: F2, ) -> ConfBuilder<S, F2>

Set the config logger used in this parse This enables introspection on the config process, so that it can be determined which program options are configured via which value sources.

pub fn parse(self) -> S

Parse based on supplied sources (or falling back to defaults), and exiting the program with errors logged to stderr if parsing fails.

pub fn try_parse(self) -> Result<S, Error>

Try to parse an instance based on supplied sources (or falling back to defaults), returning an error if parsing fails.

Source

impl<S, F> ConfBuilder<S, F>
where S: ConfSerde, F: FnMut(&dyn ConfigEvent),

pub fn doc<'de, D: Deserializer<'de>>( self, document_name: impl Into<String>, deserializer: D, ) -> ConfSerdeBuilder<'de, D, S, F>

Set the document used in this parse.

Requires a name for the document and a serde Deserializer representing the content.

The name is typically the name of a file, and is used in error messages.

The deserializer is, for example, a serde_json::Value, serde_yaml::Value, figment::Value, etc. which you have already loaded from disk and parsed in an unstructured way.